#pragma once

#include <windows.h>
#include "types.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef STRIKE7API_EXPORTS
#define STRIKE7API __declspec(dllexport)
#else
#define STRIKE7API __declspec(dllimport)
#endif

typedef uint16_t AppletId_t;

// event callbacks
typedef void (__stdcall *AppletActiveEventFunc)( uint16_t appletId, uint16_t endpointId, uint32_t endpointType );
typedef void (__stdcall *AppletInactiveEventFunc)( uint16_t appletId, uint16_t endpointId );
typedef void (__stdcall *AppletTouchEventFunc)( uint16_t appletId, uint16_t endpointId, uint32_t x, uint32_t y, BOOL pressed );
typedef void (__stdcall *AppletResetEventFunc)( uint16_t appletId, uint16_t endpointId );

// api functions


// event registration

STRIKE7API
void RegisterAppletActiveEvent( AppletActiveEventFunc, AppletId_t appletId );

STRIKE7API
void RegisterAppletInactiveEvent( AppletInactiveEventFunc, AppletId_t appletId );

STRIKE7API
void RegisterAppletTouchEvent( AppletTouchEventFunc, AppletId_t appletId );

STRIKE7API
void RegisterAppletResetEvent( AppletResetEventFunc, AppletId_t appletId );

// applet management

STRIKE7API
AppletId_t OpenApplet( const TCHAR* name, uint32_t iconWidth, uint32_t iconHeight, void *iconRGBA, uint32_t supportedDevices[]);

STRIKE7API
void CloseApplet( AppletId_t appletId );

// drawing functions

STRIKE7API
void AppletDraw( AppletId_t appletId, uint16_t endpointId, int32_t x, int32_t y, uint32_t w, uint32_t h, void *data );

STRIKE7API
void AppletFill( AppletId_t appletId, uint16_t endpointId, int32_t x, int32_t y, uint32_t w, uint32_t h, uint32_t colour );

STRIKE7API
void AppletDrawText( AppletId_t appletId, uint16_t endpointId, int32_t x, int32_t y, uint32_t size, uint32_t colour, const TCHAR* text );

STRIKE7API
void AppletScroll( AppletId_t appletId, uint16_t endpointId, uint32_t x, uint32_t y, uint32_t w, uint32_t height, int32_t dx, int32_t dy );

STRIKE7API
void AppletUpdateScreen( uint16_t appletId, uint16_t endpointId );

#ifdef __cplusplus
}
#endif
