
#include "stdafx.h"
#include "mapApplet.h"

using namespace MCStrike7;
using namespace Gdiplus;

#define TIMER_INTERVAL 200

int m_viewWidth = 480;
int m_viewHeight = 320;

static MapApplet *g_mapApplet = NULL;

struct Lock
{
	HANDLE semaphore;
	DWORD waitResult;

	Lock(){}

	Lock(HANDLE _semaphore)
		: semaphore(_semaphore),
		  waitResult(WaitForSingleObject( semaphore, 0L ))
	{}

	~Lock()
	{
		if (waitResult == WAIT_OBJECT_0)
		{
			ReleaseSemaphore( semaphore, 1, NULL );      
		}
	}
};

MapApplet::MapApplet(Bitmap *map)
{
	g_mapApplet = this; 

	m_x = 0;
	m_y = 0;

	m_semaphore = CreateSemaphore( NULL, 0, 0, NULL);          

	m_map = map;
	m_mapWidth = m_map->GetWidth();
	m_mapHeight = m_map->GetHeight();

    Bitmap icon(L"..\\C#\\Resources\\MadCatzIcon.png");

	BitmapData data;
	Rect rect(0,0,icon.GetWidth(),icon.GetHeight());
	icon.LockBits(&rect, ImageLockModeRead, PixelFormat32bppARGB, &data);
	OpenApplet( _T("Map"), icon.GetWidth(), icon.GetHeight(), data.Scan0 );
	icon.UnlockBits(&data);
}

MapApplet::~MapApplet()
{
	KillTimer(NULL, 1);
	CloseHandle(m_semaphore);
}

void MapApplet::UpdateWholeMap()
{
	UpdateMapSection(0, 0, m_x, m_y, m_viewWidth, m_viewHeight);
	UpdateScreen();
}

void MapApplet::UpdateMapSection(int x, int y, int fromX, int fromY, int width, int height)
{
	// Clone a portion of the bitmap.
    Bitmap* clone = m_map->Clone(fromX, fromY, width, height, PixelFormat32bppARGB);
	
	BitmapData data;
	Rect rect(0, 0, width, height);

	clone->LockBits(&rect, ImageLockModeRead, PixelFormat32bppARGB, &data);
	Draw( x, y, width, height, data.Scan0 );
	clone->UnlockBits(&data);

	delete clone;
}

void MapApplet::ScrollMap(int dx, int dy)
{
	m_x += dx;
	m_y += dy;

	//Check bounds
	if (m_x < 0)
	{
		m_x -= dx;
		dx = m_x * -1;
		m_x = 0;
	}

	if (m_y < 0)
	{
		m_y -= dy;
		dy = m_y * -1;
		m_y = 0;
	}

	if ((m_x + m_viewWidth) > m_mapWidth)
	{
		m_x -= dx;
		dx = m_mapWidth - (m_x + m_viewWidth);
		m_x += dx;
	}

	if ((m_y + m_viewHeight) > m_mapHeight)
	{
		m_y -= dy;
		dy = m_mapHeight - (m_y + m_viewHeight);
		m_y += dy;
	}

	//Scroll
	Scroll(0, 0, m_viewWidth, m_viewHeight, dx * -1, dy * -1);

	//Repaint

	//X (width) portion of the update
	int x_startX = 0;
	int x_startY = 0;
	int x_width = 0;
	int x_height = 0;
	int x_fromX = 0;
	int x_fromY = 0;

	//Y (height) portion of the update
	int y_startX = 0;
	int y_startY = 0;
	int y_width = 0;
	int y_height = 0;
	int y_fromX = 0;
	int y_fromY = 0;

	//The little block that intersects the X&Y portions of the update
	int b_startX = 0;
	int b_startY = 0;
	int b_width = 0;
	int b_height = 0;
	int b_fromX = 0;
	int b_fromY = 0;

	//Calcualtions per quadrant
	if (dx >= 0 && dy >= 0)
	{
		b_width = dx;
		b_height = dy;
		b_startX = m_viewWidth - dx;
		b_startY = m_viewHeight - dy;
		b_fromX = m_x + b_startX;
		b_fromY = m_y + b_startY;

	}
	else if (dx >= 0 && dy < 0)
	{
		b_width = dx;
		b_height = dy * -1;
		b_startX = m_viewWidth - dx;
		b_startY = 0;
		b_fromX = m_x + b_startX;
		b_fromY = m_y;
	}
	else if (dx < 0 && dy >= 0)
	{
		b_width = dx * -1;
		b_height = dy;
		b_startX = 0;
		b_startY = m_viewHeight - b_height;
		b_fromX = m_x;
		b_fromY = m_y + b_startY;
	}
	else if (dx < 0 && dy < 0)
	{
		b_width = dx * -1;
		b_height = dy * -1;
		b_startX = 0;
		b_startY = 0;
		b_fromX = m_x;
		b_fromY = m_y;
	}

	if (dx >= 0)
	{
		x_width = b_startX;
		x_height = b_height;
		x_startX = 0;
		x_startY = b_startY;
		x_fromX = m_x;
		x_fromY = b_fromY;
	}
	else
	{
		x_width = m_viewWidth - b_width;
		x_height = b_height;
		x_startX = b_width;
		x_startY = b_startY;
		x_fromX = m_x + b_width;
		x_fromY = m_y + b_startY;
	}

	if (dy >= 0)
	{
		y_width = b_width;
		y_height = b_startY;
		y_startX = b_startX;
		y_startY = 0;
		y_fromX = b_fromX;
		y_fromY = m_y;
	}
	else
	{
		y_width = b_width;
		y_height = m_viewHeight - b_height;
		y_startX = b_startX;
		y_startY = b_height;
		y_fromX = b_fromX;
		y_fromY = m_y + b_height;
	}

	if (x_width > 0 && x_height > 0)
	{
		UpdateMapSection(x_startX, x_startY, x_fromX, x_fromY, x_width, x_height);
	}

	if (y_width > 0 && y_height > 0)
	{
		UpdateMapSection(y_startX, y_startY, y_fromX, y_fromY, y_width, y_height);
	}

	if (b_width > 0 && b_height > 0)
	{
		UpdateMapSection(b_startX, b_startY, b_fromX, b_fromY, b_width, b_height);
	}
}

void MapApplet::OnAppletActive()
{
	UpdateWholeMap();
}

void MapApplet::OnAppletInactive()
{
	KillTimer(NULL, 1001);
	//m_screenUpdateTimer.Stop();
}

void MapApplet::OnAppletReset()
{
	KillTimer(NULL, 1001);
	//m_screenUpdateTimer.Stop();
}

void MapApplet::OnAppletTouch(uint32_t x, uint32_t y, bool pressed)
{
	if (m_pressed && pressed)
	{
		Lock(m_semaphore);

		m_scrollX += m_lastTouchX - x;
		m_scrollY += m_lastTouchY - y;
		
	}

	if (m_pressed && !pressed)
	{
		//m_screenUpdateTimer.Stop();
		UpdateScreenForScroll();
	}

	if (!m_pressed && pressed)
	{
		SetTimer(NULL, 1001, TIMER_INTERVAL, TimerProc);
		//m_screenUpdateTimer.Start();
	}

	m_pressed = pressed;
	m_lastTouchX = x;
	m_lastTouchY = y;
}

void MapApplet::UpdateScreenForScroll()
{
	int scrollX;
	int scrollY;

	{
		Lock(m_semaphore);

		scrollX = m_scrollX;
		scrollY = m_scrollY;

		m_scrollX = 0;
		m_scrollY = 0;
	}

	if (scrollX + scrollY == 0) return;

	{
		Lock(semaphore);

		ScrollMap(scrollX, scrollY);

		//Sends the screen update command.
		UpdateScreen();
	}
}

void MapApplet::Event_ScreenUpdateTimer_Elapsed()
{
//	UpdateScreenForScroll();
}

/* The Timer Procedure */
VOID CALLBACK MapApplet::TimerProc(HWND hwnd, UINT uMsg,  UINT_PTR idEvent, DWORD dwTime)
{
	g_mapApplet->UpdateScreenForScroll();	
}